from PIL import Image
import cherrypy
import tinydb
import jinja2
import os


class Gyf:

    def __init__(self):
        if not os.path.exists(os.path.expanduser('~/.gyf')):
            os.mkdir(os.path.expanduser('~/.gyf'))
        if not os.path.exists(os.path.expanduser('~/.gyf/images')):
            os.mkdir(os.path.expanduser('~/.gyf/images'))
        self.env = jinja2.Environment(loader=jinja2.PackageLoader('gyf', 'templates'))
        self.db = tinydb.TinyDB(os.path.expanduser('~/.gyf/db.json'))

    @cherrypy.expose
    def index(self):
        template = self.env.get_template('home.html')
        return template.render(images=self.db.all(), folder=os.path.join(os.path.expanduser("~"), ".gyf", "images"))

    @cherrypy.expose
    def add(self):
        #self.db.insert({'name': 'Test'})
        template = self.env.get_template('add.html')
        return template.render()

    @cherrypy.expose
    def upload(self, name, file):
        file_path = os.path.join(os.path.expanduser('~/.gyf/images'), file.filename)
        size = 0
        all_data = bytearray()
        while True:
            data = file.file.read(8192)
            all_data += data
            if not data:
                break
            size += len(data)
        saved_file = open(file_path, 'wb')
        saved_file.write(all_data)
        saved_file.close()

        im = Image.open(file_path)
        im.thumbnail([256, 256])
        im.save(file_path + ".thumbnail.gif", "GIF")

        width, height = im.size

        self.db.insert({'name': name, 'file': file.filename, 'width': width, 'height': height})

        raise cherrypy.HTTPRedirect("/")


def cli():
    conf = {
        '/': {
            'tools.sessions.on': True,
            'tools.staticdir.root': os.path.dirname(os.path.realpath(__file__))
        },
        '/static': {
            'tools.staticdir.on': True,
            'tools.staticdir.dir': 'public'
        },
        '/images': {
            'tools.staticdir.on': True,
            'tools.staticdir.dir': os.path.join(os.path.expanduser("~"), ".gyf", "images")
        }
    }
    cherrypy.quickstart(Gyf(), '/', conf)


if __name__ == '__main__':
    cli()